﻿using IRaCIS.Core.Domain.Share;
using System;
using System.Collections.Generic;
using System.ComponentModel.DataAnnotations;
using System.Text;

namespace IRaCIS.Core.Application.Contracts
{

    public class BasicTrialConfig
    {
        public Guid TrialId { get; set; }

        /// <summary>
        /// 受试者编号具体规则
        /// </summary>
        public string SubjectCodeRule { get; set; } = string.Empty;

        public List<Guid> TrialCriterionIds { get; set; } = new List<Guid>();

        public List<string> TrialCriterionNames { get; set; } = new List<string>();

        public List<Guid> ClinicalDataTrialSetIds { get; set; } = new List<Guid>();

        public bool IsSubjectSecondCodeView { get; set; }

        /// <summary>
        /// 是否 验证拍片日期  
        /// </summary>
        public bool IsVerifyVisitImageDate { get; set; } = false;

        /// <summary>
        ///  是否 提醒受试者编号规则  
        /// </summary>
        public bool IsNoticeSubjectCodeRule { get; set; } = true;

        /// <summary>
        /// 是否 有基准时间（首次给药时间）
        /// </summary>
        public bool IsHaveFirstGiveMedicineDate { get; set; } = true;

        /// <summary>
        /// 是否有  受试者年龄
        /// </summary>
        public bool IsHaveSubjectAge { get; set; } = false;

        public bool IsTrialBasicLogicConfirmed { get; set; } = false;


        /// <summary>
        /// 出组后计划外访视名称
        /// </summary>
        public string OutEnrollmentVisitName { get; set; } = "EOT";

        /// <summary>
        /// 临床信息传输 1：系统录入2：系统录入+PDF 0：无
        /// </summary>
        public int ClinicalInformationTransmissionEnum { get; set; } = 1;

        public bool IsSubjectSexView { get; set; } = false;


        public int ChangeDefalutDays { get; set; } = 5;


        /// <summary>
        /// 跨项目复制
        /// </summary>
        public bool IsImageReplicationAcrossTrial { get; set; } = false;


        public string BodyPartTypes { get; set; } = string.Empty;


        public string Modalitys { get; set; } = string.Empty;


    }

    public class TrialProcessConfig
    {

        public List<Guid> CriterionIds { get; set; } = new List<Guid>();

        public List<Guid> TrialCriterionIds { get; set; } = new List<Guid>();


        public Guid TrialId { get; set; }


        /// <summary>
        /// QC流程  0 不审，1 单审，2双审  
        /// </summary>
        public TrialQCProcess QCProcessEnum { get; set; }

        /// <summary>
        /// 影像一致性核查
        /// </summary>
        public bool IsImageConsistencyVerification { get; set; } = false;

        /// <summary>
        /// 流程是否确认
        /// </summary>
        public bool IsTrialProcessConfirmed { get; set; } = false;

        /// <summary>
        /// 1 Mint、2 PACS  
        /// </summary>

        public int ImagePlatform { get; set; } = 1;

        //阅片方式
        public int ReadingMode { get; set; }

        //阅片类型 
        public int ReadingType { get; set; }


        public bool IsGlobalReading { get; set; } = true;

        public bool? IsArbitrationReading { get; set; }

        public bool? IsClinicalReading { get; set; }


        public int ArbitrationRule { get; set; }

        public int? DigitPlaces { get; set; }


    }


    public class TrialTaskConfig
    {
        public Guid TrialId { get; set; }
        public TaskAllocateObj TaskAllocateObjEnum { get; set; }

        public bool IsFollowVisitAutoAssign { get; set; }

        public bool IsFollowGlobalVisitAutoAssign { get; set; }

        public TaskAllocateDefaultState FollowVisitAutoAssignDefaultState { get; set; }

        public TaskAllocateDefaultState FollowGlobalVisitAutoAssignDefaultState { get; set; }



    }

    public class TrialBasicConfigView : TrialTaskConfigView
    {
        /// <summary>
        /// QC流程  0 不审，1 单审，2双审  
        /// </summary>
        public TrialQCProcess QCProcessEnum { get; set; }

        /// <summary>
        /// 影像一致性核查
        /// </summary>
        public bool IsImageConsistencyVerification { get; set; } = false;

        /// <summary>
        /// 流程是否确认
        /// </summary>
        public bool IsTrialProcessConfirmed { get; set; } = false;

       

        //阅片方式
        public int ReadingMode { get; set; }

        //阅片类型 
        public int ReadingType { get; set; }


        public bool IsGlobalReading { get; set; } = true;

        public bool? IsArbitrationReading { get; set; }

        public bool? IsClinicalReading { get; set; }


        public int ArbitrationRule { get; set; }

     
    }

    public class TrialTaskConfigView : TrialTaskConfig
    {
        public ReadingTaskViewMethod ReadingTaskViewEnum { get; set; }

        public bool IsReadingTaskViewInOrder { get; set; } = true;

        public bool IsFollowJudgeTaskAutoAssign { get; set; }

        /// <summary>
        /// 阅片是否显示受试者信息
        /// </summary>
        public bool IsReadingShowSubjectInfo { get; set; } 

        /// <summary>
        /// 阅片是否显示既往结果
        /// </summary>
        public bool IsReadingShowPreviousResults { get; set; } 

        public TaskAllocateDefaultState FollowJudgeTaskAutoAssignDefaultState { get; set; }
    }

    public class TrialReadingTaskViewConfig
    {
        public Guid TrialId { get; set; }

        public ReadingTaskViewMethod ReadingTaskViewEnum { get; set; }

        public bool IsReadingTaskViewInOrder { get; set; } = true;

        /// <summary>
        /// 阅片是否显示受试者信息
        /// </summary>
        public bool IsReadingShowSubjectInfo { get; set; }

        /// <summary>
        /// 阅片是否显示既往结果
        /// </summary>
        public bool IsReadingShowPreviousResults { get; set; } 
    }

    public class TrialJudgeTaskConfig
    {

        public Guid TrialId { get; set; }

        public bool IsFollowJudgeTaskAutoAssign { get; set; }

        public TaskAllocateDefaultState FollowJudgeTaskAutoAssignDefaultState { get; set; }
    }

    public class TrialUrgentConfig
    {
        public Guid TrialId { get; set; }

        public bool IsSubjectExpeditedView { get; set; } = false;

        /// <summary>
        /// 是否有 入组评估确认
        /// </summary>
        public bool IsEnrollementQualificationConfirm { get; set; } = false;

        /// <summary>
        /// 流程配置是否确认
        /// </summary>
        public bool IsTrialUrgentConfirmed { get; set; } = false;

        public bool IsUrgent { get; set; }

        public bool IsPDProgressView { get; set; }

    }

    public class TrialStateChangeDTO
    {
        public Guid Id { get; set; }
        public Guid TrialId { get; set; }


        public string OriginState { get; set; } = String.Empty;


        public string NowState { get; set; } = String.Empty;


        public string Reason { get; set; } = String.Empty;


        public DateTime CreateTime { get; set; }


        public Guid CreateUserId { get; set; }

        public string UserName { get; set; } = String.Empty;

        public string UserRealName { get; set; } = String.Empty;

        public string SignCode { get; set; } = string.Empty;
    }



    public class SignConfirmDTO
    {
        [NotDefault]
        public Guid TrialId { get; set; }

        //[NotDefault]
        //public Guid SignId { get; set; }


        [NotDefault]
        public string SignCode { get; set; } = string.Empty;
    }

    public class TrialConfigDTO : BasicTrialConfig
    {


        public List<Guid> CriterionIds { get; set; } = new List<Guid>();

        public List<string> ClinicalDataSetNames { get; set; } = new List<string>();

        public bool IsTrialProcessConfirmed { get; set; }


        public bool IsTrialUrgentConfirmed { get; set; }


        public int DigitPlaces { get; set; }



        /// <summary>
        /// QC流程  0 不审，1 单审，2双审  
        /// </summary>
        public TrialQCProcess QCProcessEnum { get; set; }

        /// <summary>
        /// 影像一致性核查
        /// </summary>
        public bool IsImageConsistencyVerification { get; set; } = false;

        /// <summary>
        /// 1 Mint、2 PACS  
        /// </summary>

        public int ImagePlatform { get; set; } = 1;

        //阅片方式
        public int ReadingMode { get; set; }

        //阅片类型 
        public int ReadingType { get; set; }


        public bool IsGlobalReading { get; set; } = true;

        public bool? IsArbitrationReading { get; set; }

        public bool? IsClinicalReading { get; set; }


        public int ArbitrationRule { get; set; }





        public bool IsSubjectExpeditedView { get; set; } = false;

        /// <summary>
        /// 是否有 入组评估确认
        /// </summary>
        public bool IsEnrollementQualificationConfirm { get; set; } = false;

        public bool IsUrgent { get; set; }

        public bool IsPDProgressView { get; set; }



        //public bool IsTrialStart { get; set; } = false;

        public bool IsQCQuestionConfirmed { get; set; }

        public string TrialStatusStr { get; set; } = string.Empty;




        //public string DocumentConfirmSignText { get; set; } = string.Empty;
        //public string ImageQCSignText { get; set; } = string.Empty;
        //public string PreliminaryAuditReuploadText { get; set; } = string.Empty;
        //public string ReviewAuditReuploadText { get; set; } = string.Empty;
        //public string CheckBackText { get; set; } = string.Empty;
        //public string CheckCloseText { get; set; } = string.Empty;
        //public string ChallengeTypes { get; set; } = string.Empty;
        //public bool IsImageExport { get; set; } = false;
        //public bool IsCRAAuditClinicalInformation { get; set; } = false;
        //public string TrialSiteSurveyUserRoles { get; set; } = string.Empty;
        //public string TrialSiteSurveyEquipmentType { get; set; } = string.Empty;

    }
}
